'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _index = require('../index');

var _index2 = _interopRequireDefault(_index);

var _log = require('../util/log');

var _actions = require('../actions');

var _actions2 = _interopRequireDefault(_actions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Command line interface for Sagui
 */
exports.default = function () {
  var argv = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

  try {
    _commander2.default.parse(argv);
  } catch (e) {
    handleError(e);
  }
};

var handleError = function handleError(e) {
  if (e instanceof _index.SaguiPath || e instanceof _index.MissingPackageJSON) {
    (0, _log.logWarning)(e.message);
    return;
  }

  if (e instanceof _index.InvalidSaguiConfig) {
    (0, _log.logError)(e.message);
  } else if (e) {
    (0, _log.logError)(e.stack || e);
  }

  process.exit(1);
};

var setupAction = function setupAction(NODE_ENV, actionsToRun) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return function () {
    var cliOptions = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    if (!process.env.NODE_ENV || process.env.NODE_ENV && process.env.NODE_ENV.trim().length === 0) {
      process.env.NODE_ENV = NODE_ENV;
      (0, _log.log)(`Setting NODE_ENV=${NODE_ENV}`);
    } else {
      (0, _log.log)(`Using NODE_ENV=${NODE_ENV}`);
    }

    var tasks = actionsToRun.map(function (action) {
      return function () {
        return (0, _index2.default)(_extends({}, options, cliOptions, {
          action,
          projectPath: process.env.SAGUI_LINK ? process.cwd() : _path2.default.join(__dirname, '../../../../')
        }));
      };
    });

    tasks.reduce(function (previous, next) {
      return previous.then(next);
    }, Promise.resolve()).then(function () {
      return process.exit(0);
    }, handleError);
  };
};

_commander2.default.command('format').description('Format the source code').action(setupAction('development', [_actions2.default.FORMAT]));

_commander2.default.command('develop').description('Run development environment').option('-p, --port <n>', 'Port the server will listen (default: 3000)', parseInt).action(setupAction('development', [_actions2.default.DEVELOP]));

_commander2.default.command('build').description('Build based on the DEVELOPMENT environment').option('-p, --optimize', 'Optimize the build (minify, dedup...)').action(setupAction('development', [_actions2.default.BUILD]));

_commander2.default.command('dist').description('Build and optimize a ready for PRODUCTION distribution').action(setupAction('production', [_actions2.default.BUILD], { optimize: true }));

_commander2.default.command('test').description('Run all the test checks (with coverage enabled)').action(setupAction('test', [_actions2.default.TEST_LINT, _actions2.default.TEST_TYPECHECK, _actions2.default.TEST_UNIT]), { coverage: true });

_commander2.default.command('test:unit').description('Run unit tests').option('-w, --watch', 'Run tests on any file change').option('-c, --coverage', 'Generate a coverage report').action(setupAction('test', [_actions2.default.TEST_UNIT]));

_commander2.default.command('test:lint').description('Lint the code').action(setupAction('test', [_actions2.default.TEST_LINT]));

_commander2.default.command('test:typecheck').description('Typecheck the code').action(setupAction('test', [_actions2.default.TEST_TYPECHECK]));

_commander2.default.command('update').description('Update Sagui in the current project').action(setupAction('development', [_actions2.default.UPDATE]));

_commander2.default.action(function () {
  (0, _log.logError)(`This action is not valid. Use --help to learn more about all the valid actions`);
  process.exit(1);
});