'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _mergeKarma = require('../util/merge-karma');

var _mergeKarma2 = _interopRequireDefault(_mergeKarma);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

exports.default = function () {
  var saguiConfig = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var webpackConfig = arguments[1];

  return (0, _mergeKarma2.default)(buildStandardKarmaConfig(saguiConfig, webpackConfig), saguiConfig.additionalKarmaConfig);
};

var buildStandardKarmaConfig = function buildStandardKarmaConfig(saguiConfig, webpackConfig) {
  var projectPath = saguiConfig.projectPath,
      watch = saguiConfig.watch,
      coverage = saguiConfig.coverage,
      _saguiConfig$addition = saguiConfig.additionalKarmaConfig,
      additionalKarmaConfig = _saguiConfig$addition === undefined ? {} : _saguiConfig$addition;


  return {
    // Webpack itself supports an array of configurations
    // and although harmless to have them all running the tests
    // it is not required and only produces double execution
    webpack: Array.isArray(webpackConfig) ? webpackConfig[0] : webpackConfig,

    basePath: projectPath,

    browsers: ['PhantomJS'],

    reporters: ['mocha'].concat(_toConsumableArray(coverage ? ['coverage'] : [])),

    frameworks: ['jasmine', 'detectBrowsers'],

    detectBrowsers: {
      enabled: additionalKarmaConfig.browsers && additionalKarmaConfig.browsers > 0,
      usePhantomJS: true,

      postDetection: function postDetection(availableBrowser) {
        var isTravis = process.env['TRAVIS'] === 'true';
        var isChromeAvailable = availableBrowser.indexOf('Chrome') > -1;
        var isTravisChromeAvailable = (process.env['TRAVIS_STACK_FEATURES'] || '').indexOf('google-chrome') !== -1;

        if (isChromeAvailable && (!isTravis || isTravis && isTravisChromeAvailable)) {
          return ['ChromeHeadless'];
        }

        return ['PhantomJS'];
      }
    },

    // first run will have the full output and
    // the next runs just output the summary and
    // errors in mocha style
    mochaReporter: {
      output: 'autowatch'
    },

    coverageReporter: {
      dir: _path2.default.join(projectPath, 'coverage'),
      reporters: [{ type: 'html', subdir: 'html' },

      // outputs the coverage report in the console
      { type: 'text' }]
    },

    // we don't watch any of the files here,
    // since that is taken care by the preprocessors
    files: [{ pattern: 'node_modules/sagui/karma-static-files/test-bundle.js', watched: false }, { pattern: 'src/**/*', watched: false, included: false }],

    preprocessors: {
      // add webpack as preprocessor
      'node_modules/sagui/karma-static-files/test-bundle.js': ['webpack', 'sourcemap']
    },

    webpackServer: {
      noInfo: true
    },

    singleRun: !watch,
    autoWatch: watch
  };
};