'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _actions = require('../actions');

var _actions2 = _interopRequireDefault(_actions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Defines how to build a single library.
 *
 * The library parameter is both the name of the file
 * is the src/ directory and also the name of the built
 * artefact.
 */
exports.default = function (library, _ref) {
  var projectPath = _ref.projectPath,
      action = _ref.action;

  if (!library || action === _actions2.default.TEST_UNIT) {
    return {};
  }

  var externals = probeExternals(projectPath);

  var libraryConfig = typeof library === 'string' ? { main: library } : library;

  var target = libraryConfig.umdName ? 'umd' : 'commonjs2';

  return {
    entry: `./${libraryConfig.main}.js`,
    output: {
      path: (0, _path.join)(projectPath, 'dist'),
      filename: `${libraryConfig.main}.js`,
      libraryTarget: target,
      library: libraryConfig.umdName
    },
    externals: externals
  };
};

function probeExternals(projectPath) {
  var projectPackageJSON = require((0, _path.join)(projectPath, 'package.json'));
  return Object.keys(projectPackageJSON.peerDependencies || {});
}