'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _htmlWebpackPlugin = require('html-webpack-plugin');

var _htmlWebpackPlugin2 = _interopRequireDefault(_htmlWebpackPlugin);

var _path = require('path');

var _webpack = require('webpack');

var _actions = require('../actions');

var _actions2 = _interopRequireDefault(_actions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function () {
  var pages = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var _ref = arguments[1];
  var action = _ref.action,
      _ref$chunks = _ref.chunks,
      chunks = _ref$chunks === undefined ? {} : _ref$chunks,
      projectPath = _ref.projectPath;

  if (pages.length === 0 || action === _actions2.default.TEST_UNIT) {
    return {};
  }

  var entry = configureEntry(pages);
  var plugins = configurePlugins(pages, chunks);
  var filenamePattern = action === _actions2.default.BUILD ? '[name]-[chunkhash]' : '[name]'; // For better performance during development

  return {
    output: {
      path: (0, _path.join)(projectPath, 'dist'),
      filename: `${filenamePattern}.js`,
      chunkFilename: `${filenamePattern}.chunk.js`
    },
    plugins,
    entry
  };
};

function configureEntry(pages) {
  var entry = {};

  pages.forEach(function (page) {
    var pageName = getPageName(page);
    entry[pageName] = [`./${pageName}`];
  });

  return entry;
}

function configurePlugins(pages, chunksConfig) {
  var plugins = pages.map(function (page) {
    var pageName = getPageName(page);
    var chunks = Object.keys(chunksConfig).reduce(function (chunks, key) {
      if (chunksConfig[key]) {
        chunks.unshift(key);
      }
      return chunks;
    }, [pageName]);

    return new _htmlWebpackPlugin2.default({
      template: `${pageName}.html`,
      filename: `${pageName}.html`,
      chunks
    });
  });

  if (pages.length > 1) {
    var chunks = pages.reduce(function (chunks, page) {
      if (typeof page === 'string') {
        chunks.push(page);
      }
      if (page.independent !== true) {
        chunks.push(page.name);
      }
      return chunks;
    }, []);

    if (chunksConfig.vendor) {
      plugins.push(new _webpack.optimize.CommonsChunkPlugin({
        name: 'vendor',
        minChunks: function minChunks(module) {
          return (/node_modules/.test(module.context)
          );
        },
        chunks
      }));
    }
    if (chunksConfig.common) {
      plugins.push(new _webpack.optimize.CommonsChunkPlugin({
        name: 'common',
        minChunks: 2,
        chunks
      }));
    }
  }

  return plugins;
}

function getPageName(page) {
  return typeof page === 'string' ? page : page.name;
}