'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _caseSensitivePathsWebpackPlugin = require('case-sensitive-paths-webpack-plugin');

var _caseSensitivePathsWebpackPlugin2 = _interopRequireDefault(_caseSensitivePathsWebpackPlugin);

var _cleanWebpackPlugin = require('clean-webpack-plugin');

var _cleanWebpackPlugin2 = _interopRequireDefault(_cleanWebpackPlugin);

var _webpackMerge = require('webpack-merge');

var _webpackMerge2 = _interopRequireDefault(_webpackMerge);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _webpackMd5Hash = require('webpack-md5-hash');

var _webpackMd5Hash2 = _interopRequireDefault(_webpackMd5Hash);

var _actions = require('../actions');

var _actions2 = _interopRequireDefault(_actions);

var _fileExtensions = require('../file-extensions');

var _fileExtensions2 = _interopRequireDefault(_fileExtensions);

var _buildLibraryConfig = require('./build-library-config');

var _buildLibraryConfig2 = _interopRequireDefault(_buildLibraryConfig);

var _buildPagesConfig = require('./build-pages-config');

var _buildPagesConfig2 = _interopRequireDefault(_buildPagesConfig);

var _loaders = require('./loaders');

var _loaders2 = _interopRequireDefault(_loaders);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/**
 * Takes a saguiConfig object and construct a webpack configuration object
 * It support two different types of archetypes; pages and libraries
 *
 * @return ready to use webpack configuration as an array
 */
exports.default = function () {
  var saguiConfig = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _saguiConfig$pages = saguiConfig.pages,
      pages = _saguiConfig$pages === undefined ? [] : _saguiConfig$pages,
      _saguiConfig$librarie = saguiConfig.libraries,
      libraries = _saguiConfig$librarie === undefined ? [] : _saguiConfig$librarie,
      sharedSaguiConfig = _objectWithoutProperties(saguiConfig, ['pages', 'libraries']);

  var sharedWebpackConfig = _webpackMerge2.default.smart(buildSharedWebpackConfig(saguiConfig), (0, _loaders2.default)(saguiConfig), sharedSaguiConfig.additionalWebpackConfig);

  return [].concat(_toConsumableArray(pages.length > 0 ? [(0, _buildPagesConfig2.default)(pages, sharedSaguiConfig)] : []), _toConsumableArray(libraries.map(function (libraryConfig) {
    return (0, _buildLibraryConfig2.default)(libraryConfig, sharedSaguiConfig);
  }))).map(function (entryPointWebpackConfig) {
    return _webpackMerge2.default.smart(sharedWebpackConfig, entryPointWebpackConfig);
  });
};

var buildSharedWebpackConfig = function buildSharedWebpackConfig(saguiConfig) {
  var action = saguiConfig.action,
      projectPath = saguiConfig.projectPath,
      saguiPath = saguiConfig.saguiPath,
      watch = saguiConfig.watch,
      optimize = saguiConfig.optimize;


  var projectSourcePath = _path2.default.join(projectPath, 'src');

  return {
    context: projectSourcePath,

    target: 'web',

    devtool: action === _actions2.default.BUILD ? 'source-map' : 'inline-source-map',

    plugins: [].concat(_toConsumableArray(watch || action === _actions2.default.DEVELOP ? [] : [new _webpack.NoEmitOnErrorsPlugin()]), _toConsumableArray(optimize ? [new _webpack2.default.optimize.UglifyJsPlugin({
      sourceMap: true
    }),

    // signal loaders to minimize
    // https://webpack.js.org/guides/migrating/#uglifyjsplugin-minimize-loaders
    new _webpack2.default.LoaderOptionsPlugin({
      minimize: true
    })] : []), _toConsumableArray(action === _actions2.default.DEVELOP ? [new _webpack.HotModuleReplacementPlugin()] : []), _toConsumableArray(action === _actions2.default.BUILD ? [new _webpackMd5Hash2.default()] : []), _toConsumableArray(action === _actions2.default.BUILD ? [new _cleanWebpackPlugin2.default(['dist'], {
      root: projectPath,
      verbose: false
    })] : []), [new _caseSensitivePathsWebpackPlugin2.default(), new _webpack.DefinePlugin({
      'process.env.NODE_ENV': JSON.stringify(process.env.NODE_ENV)
    })]),

    // In wepback 2 builtin node.js modules take precedence over the normal modules
    // So, if a user creates a `constants` module and tries to require it
    // with `import 'constants'`, the node.js module will be imported instead
    //
    // This configuration disables this behavior
    // More information: https://github.com/webpack/webpack/issues/4666#issuecomment-292700060
    node: {
      constants: false
    },

    resolve: {
      extensions: _fileExtensions2.default.list.JAVASCRIPT,
      modules: [
      // Keep same behavior as Node.js module resolution:
      // - Precedence: `node_modules` win over `src`;
      // - Scan modules by looking through the current directory and its ancestors.
      'node_modules', projectSourcePath,

      // Sagui node_modules is required in the path to be able
      // to load the `webpack-hot-middleware`
      _path2.default.join(saguiPath, '/node_modules')],

      // In order for `npm link` and `yarn link` to work, Webpack should not
      // resolve symlinks inside the `node_modules` folder: otherwise, it will
      // try to load the linked dependencies as if they were part of the
      // project code, even if they have a different babel/eslint/etc
      // configuration.
      symlinks: false
    },

    resolveLoader: {
      // Should first try to resolve loaders nested within Sagui.
      modules: [_path2.default.join(saguiPath, '/node_modules'), _path2.default.join(projectPath, '/node_modules')]
    }
  };
};