'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _webpack = require('webpack');

var _happypack = require('happypack');

var _happypack2 = _interopRequireDefault(_happypack);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fileExtensions = require('../../file-extensions');

var _fileExtensions2 = _interopRequireDefault(_fileExtensions);

var _actions = require('../../actions');

var _actions2 = _interopRequireDefault(_actions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

exports.default = {
  name: 'javaScript',
  configure(_ref) {
    var action = _ref.action,
        projectPath = _ref.projectPath,
        _ref$javaScript = _ref.javaScript,
        javaScript = _ref$javaScript === undefined ? {} : _ref$javaScript,
        coverage = _ref.coverage,
        browsers = _ref.browsers;

    return {
      plugins: [new _happypack2.default({
        id: 'babel',
        cache: false,
        verbose: false,
        tempDir: _path2.default.resolve(projectPath, '.sagui/happypack'),
        loaders: [{
          path: 'babel-loader',
          query: {
            babelrc: false,
            cacheDirectory: _path2.default.resolve(projectPath, '.sagui/babel'),
            presets: [[require.resolve('babel-preset-env'), {
              // Replaces require("babel-polyfill")
              // with only the polyfills you need
              // for the target browsers
              useBuiltIns: true,

              // Disables ES6 module transformation
              // which Webpack2 can understand
              // except while running tests
              // to support inject-loader
              modules: action === _actions2.default.TEST_UNIT ? 'commonjs' : false,

              targets: {
                // Unfortunately we are bound to what UglifyJS
                // currently supports as language features
                // https://github.com/babel/babel-preset-env#targetsuglify
                uglify: true,

                browsers
              }
            }], require.resolve('babel-preset-flow'), require.resolve('babel-preset-react'), require.resolve('babel-preset-stage-3')],
            plugins: [[require.resolve('babel-plugin-syntax-dynamic-import'), {}]].concat(_toConsumableArray(action === _actions2.default.DEVELOP || action === _actions2.default.TEST_UNIT ? [[require.resolve('babel-plugin-transform-react-jsx-source'), {}], [require.resolve('babel-plugin-transform-react-jsx-self'), {}]] : []), _toConsumableArray(action === _actions2.default.TEST_UNIT && coverage ? [[require.resolve('babel-plugin-istanbul'), {
              exclude: ['**/*.spec.*', '**/node_modules/**/*']
            }]] : []))
          }
        }]
      })].concat(_toConsumableArray(action === _actions2.default.DEVELOP ? [new _webpack.HotModuleReplacementPlugin()] : [])),

      module: {
        rules: [{
          test: _fileExtensions2.default.test.JAVASCRIPT,
          enforce: 'pre',
          loader: 'eslint-loader',
          exclude: /node_modules/,
          options: {
            configFile: _path2.default.join(projectPath, '.eslintrc'),
            useEslintrc: false,
            cwd: projectPath,

            // While in development make the eslint messages non-intrusive
            // by turning them all into warnings
            emitWarning: action === _actions2.default.DEVELOP
          }
        }, {
          test: _fileExtensions2.default.test.JAVASCRIPT,
          exclude: javaScript.transpileDependencies && javaScript.transpileDependencies.length > 0 ? buildExcludeCheck(javaScript.transpileDependencies) : /node_modules/,
          loader: 'happypack/loader?id=babel'
        }]
      }
    };
  }
};

/**
 * Take into consideration if the user wants any dependency
 * to be transpiled with Babel and returns an exclude check
 * function that can be used by Webpack
 */

function buildExcludeCheck() {
  var transpileDependencies = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

  var dependencyChecks = transpileDependencies.map(function (dependency) {
    return new RegExp(`node_modules.${dependency}`);
  });

  // see: https://webpack.js.org/configuration/module/#condition
  return function (assetPath) {
    var shouldTranspile = dependencyChecks.reduce(function (result, check) {
      return result || assetPath.match(check);
    }, false);

    if (shouldTranspile) {
      return false;
    }
    return !!assetPath.match(/node_modules/);
  };
}