'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _autoprefixer = require('autoprefixer');

var _autoprefixer2 = _interopRequireDefault(_autoprefixer);

var _fileExtensions = require('../../file-extensions');

var _fileExtensions2 = _interopRequireDefault(_fileExtensions);

var _actions = require('../../actions');

var _actions2 = _interopRequireDefault(_actions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var defaultConfig = {
  cssModules: true,
  sourceMaps: false,
  extract: true

  /**
   * Style preset with
   * - CSS Modules support
   * - Sass language
   * - Autoprefixer
   */
};exports.default = {
  name: 'style',
  configure(_ref) {
    var action = _ref.action,
        optimize = _ref.optimize,
        pages = _ref.pages,
        projectPath = _ref.projectPath,
        _ref$style = _ref.style,
        style = _ref$style === undefined ? {} : _ref$style,
        browsers = _ref.browsers;

    // Use null-loader during tests
    // for better performance
    if (action === _actions2.default.TEST_UNIT) {
      return {
        module: {
          rules: [{
            test: _fileExtensions2.default.test.CSS,
            loader: 'null-loader'
          }, {
            test: _fileExtensions2.default.test.SCSS,
            loader: 'null-loader'
          }]
        }
      };
    }

    var projectSourcePath = _path2.default.join(projectPath, 'src');

    var config = _extends({}, defaultConfig, style);

    var shouldExtract = config.extract && pages.length > 0 && action === _actions2.default.BUILD;
    var localIdentName = optimize ? '[hash:base64:5]' : '[path][local]-[hash:base64:5]';

    var extractSass = new _extractTextWebpackPlugin2.default('[name]-[hash]-0.css');
    var extractCss = new _extractTextWebpackPlugin2.default('[name]-[hash]-1.css');

    // toggle source maps and CSS Modules
    var cssModules = config.cssModules ? 'modules' : '';
    var sourceMaps = config.sourceMaps ? 'sourceMap' : '';

    // importLoaders: use the following postcss-loader in @import statements
    // modules: enable css-modules
    var cssLoaders = [`css-loader?${cssModules}&${sourceMaps}&importLoaders=1&localIdentName=${localIdentName}`, 'postcss-loader'];

    // importLoaders: use the following sass-loader in @import statements
    // modules: enable css-modules
    var sassLoaders = [`css-loader?${cssModules}&${sourceMaps}&importLoaders=2&localIdentName=${localIdentName}`, 'postcss-loader', `sass-loader?${sourceMaps}&outputStyle=expanded&` + 'includePaths[]=' + _path2.default.resolve(projectPath, './node_modules')];

    return {
      module: {
        rules: [{
          test: _fileExtensions2.default.test.CSS,
          loader: shouldExtract ? extractCss.extract(cssLoaders) : ['style-loader'].concat(cssLoaders).join('!')
        }, {
          test: _fileExtensions2.default.test.SCSS,
          loader: shouldExtract ? extractSass.extract(sassLoaders) : ['style-loader'].concat(sassLoaders).join('!')
        }]
      },

      plugins: [new _webpack2.default.LoaderOptionsPlugin({
        debug: true,
        options: {
          // Fixes an issue with colliding CSS Modules
          // There is an integration test to validate it
          // see:
          //  - https://github.com/saguijs/sagui/issues/338
          //  - https://github.com/webpack-contrib/css-loader/issues/413#issuecomment-283944881
          context: projectSourcePath,

          postcss: [(0, _autoprefixer2.default)({ browsers })]
        }
      })].concat(_toConsumableArray(shouldExtract ? [extractCss, extractSass] : []))
    };
  }
};