'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validateConfig = validateConfig;
exports.MissingPackageJSON = MissingPackageJSON;
exports.InvalidSaguiConfig = InvalidSaguiConfig;
exports.SaguiPath = SaguiPath;

var _ajv = require('ajv');

var _ajv2 = _interopRequireDefault(_ajv);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _json = require('./util/json');

var _json2 = _interopRequireDefault(_json);

var _fileExists = require('./util/file-exists');

var _fileExists2 = _interopRequireDefault(_fileExists);

var _saguiConfigSchema = require('./sagui-config-schema');

var _saguiConfigSchema2 = _interopRequireDefault(_saguiConfigSchema);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ajv = new _ajv2.default({
  allErrors: true,
  verbose: true
});

exports.default = function (saguiConfig) {
  var projectPath = saguiConfig.projectPath;


  sanityCheck(projectPath);

  var configPath = _path2.default.join(projectPath, 'sagui.config.js');

  if (!(0, _fileExists2.default)(configPath)) {
    return {};
  }
  return validateConfig(require(configPath));
};

function sanityCheck(projectPath) {
  var packagePath = _path2.default.join(projectPath, 'package.json');
  if (!(0, _fileExists2.default)(packagePath)) throw new MissingPackageJSON();

  var packageJSON = _json2.default.read(packagePath);
  if (packageJSON.name === 'sagui') throw new SaguiPath();
}

function validateConfig(saguiConfig) {
  var isValid = ajv.validate(_saguiConfigSchema2.default, saguiConfig);
  if (!isValid) {
    throw new InvalidSaguiConfig();
  }

  return saguiConfig;
}

function MissingPackageJSON() {
  this.name = 'MissingPackageJSON';
  this.message = 'Must be executed in target project\'s package.json path';
  this.stack = new Error().stack;
}
MissingPackageJSON.prototype = Object.create(Error.prototype);
MissingPackageJSON.prototype.constructor = MissingPackageJSON;

function InvalidSaguiConfig() {
  this.name = 'InvalidSaguiConfig';
  this.message = 'Invalid configuration. Check documentation at https://github.com/saguijs/sagui#configuration.';
  this.stack = new Error().stack;
}
MissingPackageJSON.prototype = Object.create(Error.prototype);
MissingPackageJSON.prototype.constructor = MissingPackageJSON;

function SaguiPath() {
  this.name = 'SaguiPath';
  this.message = 'Sagui CLI must not be run in Sagui\'s path';
  this.stack = new Error().stack;
}
SaguiPath.prototype = Object.create(Error.prototype);
SaguiPath.prototype.constructor = SaguiPath;