'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _log = require('../util/log');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (saguiConfig, webpackConfig) {
  return new Promise(function (resolve, reject) {
    var compiler = (0, _webpack2.default)(webpackConfig);

    compiler.run(function (err, stats) {
      var softErrors = !err && stats.toJson().errors;
      var hasSoftErrors = softErrors && softErrors.length > 0;

      if (err || hasSoftErrors) {
        (0, _log.logError)('Build failed.');

        if (err) {
          console.error(err.stack || err);
          if (err.details) console.error(err.details);
        }

        if (hasSoftErrors) {
          softErrors.forEach(function (error) {
            console.error(error);
          });
        }

        reject(new Error('Build failed'));
      } else {
        (0, _log.log)('Built successful.');
        resolve();
      }
    });
  });
};