'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _log = require('../util/log');

var _webpackDevServer = require('webpack-dev-server');

var _webpackDevServer2 = _interopRequireDefault(_webpackDevServer);

var _ip = require('ip');

var _ip2 = _interopRequireDefault(_ip);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Development server
 */
exports.default = function (saguiConfig, webpackConfig) {
  return new Promise(function (resolve, reject) {
    var options = _extends({
      hot: true,
      noInfo: true,
      historyApiFallback: saguiConfig.pages && saguiConfig.pages.length === 1
    }, saguiConfig.develop);

    try {
      var server = new _webpackDevServer2.default((0, _webpack2.default)(setupHMR(webpackConfig, saguiConfig.port)), options);

      server.listeningApp.on('error', function (e) {
        if (e.code === 'EADDRINUSE') {
          (0, _log.logError)(`Server failed to started at http://localhost:${saguiConfig.port}. Port already used.`);
          reject(new Error('This port is already used'));
        }
      });

      server.listen(saguiConfig.port, '0.0.0.0', function (err) {
        if (err) {
          (0, _log.logError)(`Server failed to started at http://localhost:${saguiConfig.port}`);
          reject(err);
        } else {
          (0, _log.log)(`Server started at http://localhost:${saguiConfig.port}/webpack-dev-server/`);
        }
      });
    } catch (e) {
      (0, _log.logError)('Server failed to start.');
      (0, _log.logError)(e.stack || e);
      reject(e);
    }
  });
};

/**
 * HMR bundle setup based on code from
 * https://github.com/webpack/webpack-dev-server/blob/master/bin/webpack-dev-server.js
 */


function setupHMR(webpackConfig, port) {
  return webpackConfig.map(function (webpack) {
    return _extends({}, webpack, {
      entry: concatHMRBundle(webpack.entry, port)
    });
  });
}

function concatHMRBundle(entry, port) {
  var devClient = [require.resolve('webpack-dev-server/client/') + `?http://${_ip2.default.address()}:${port}`, 'webpack/hot/dev-server'];

  if (typeof entry === 'object' && !Array.isArray(entry)) {
    return Object.keys(entry).reduce(function (entries, key) {
      return _extends({}, entries, {
        [key]: devClient.concat(entry[key])
      });
    }, {});
  } else {
    return devClient.concat(entry);
  }
}