'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _child_process = require('child_process');

var _flowBin = require('flow-bin');

var _flowBin2 = _interopRequireDefault(_flowBin);

var _log = require('../util/log');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var errorCodes = {
  TYPECHECK_ERROR: 2
};

exports.default = function (saguiConfig) {
  return new Promise(function (resolve, reject) {
    var commandArgs = ['check', '--color=always'];

    if (saguiConfig.javaScript && saguiConfig.javaScript.typeCheckAll) {
      commandArgs.push('--all');
    }

    if (!_flowBin2.default) {
      (0, _log.logWarning)('Typecheck is not support in your platform.');
      (0, _log.logWarning)('For more information go to https://github.com/flowtype/flow-bin/issues/');
      return resolve();
    }

    try {
      (0, _child_process.execFile)(_flowBin2.default, commandArgs, { cwd: saguiConfig.projectPath }, function (err, stdout, stderr) {
        if (err) {
          (0, _log.logError)('Type check failed:\n');

          switch (err.code) {
            case errorCodes.TYPECHECK_ERROR:
              console.log(stdout);
              break;

            default:
              console.log(err);
          }

          reject(new Error('Type check failed'));
        } else {
          (0, _log.log)('Type check completed without errors');
          resolve();
        }
      });
    } catch (e) {
      reject(e);
    }
  });
};