'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (projectPath) {
  copyBase(projectPath);
  copyDotFiles(projectPath);
};

var _path = require('path');

var _fsExtra = require('fs-extra');

var _templateDirectory = require('template-directory');

var _templateDirectory2 = _interopRequireDefault(_templateDirectory);

var _deepEqual = require('deep-equal');

var _deepEqual2 = _interopRequireDefault(_deepEqual);

var _log = require('../../util/log');

var _fileExists = require('../../util/file-exists');

var _fileExists2 = _interopRequireDefault(_fileExists);

var _json = require('../../util/json');

var _json2 = _interopRequireDefault(_json);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var basePath = (0, _path.join)(__dirname, '../../../template/base');
var dotFilesPath = (0, _path.join)(__dirname, '../../../template/dot-files');

function copyBase(projectPath) {
  var srcFolder = (0, _path.join)(projectPath, 'src');

  if (!(0, _fileExists2.default)(srcFolder)) {
    var projectName = (0, _path.basename)(projectPath);

    (0, _templateDirectory2.default)(basePath, projectPath, {
      projectName: projectName
    }, { clobber: false });
  } else {
    (0, _log.logWarning)('Skipped installing files in src, folder already exists');
  }
}

function copyDotFiles(projectPath) {
  safeCopy(projectPath, '.editorconfig');
  safeCopy(projectPath, '.eslintignore');
  safeCopy(projectPath, '.eslintrc', diffCheckESLintrc);
  safeCopy(projectPath, '.flowconfig');
}

function safeCopy(projectPath, filename, diffCheck) {
  var sourceAbsolutePath = (0, _path.join)(dotFilesPath, filename);
  var destinationAbsolutePath = (0, _path.join)(projectPath, filename);

  if ((0, _fileExists2.default)(destinationAbsolutePath)) {
    if (diffCheck) diffCheck(sourceAbsolutePath, destinationAbsolutePath);
  } else {
    (0, _fsExtra.copySync)(sourceAbsolutePath, destinationAbsolutePath, { clobber: false });
  }
}

function diffCheckESLintrc(sourceAbsolutePath, destinationAbsolutePath) {
  var rawSourceContent = (0, _fsExtra.readFileSync)(sourceAbsolutePath).toString();
  var sourceContent = _json2.default.read(sourceAbsolutePath);
  var destinationContent = _json2.default.read(destinationAbsolutePath);

  if (!(0, _deepEqual2.default)(sourceContent, destinationContent)) {
    (0, _log.logWarning)(`Custom .eslintrc detected: Linting might not work as expected. Expected:`);
    (0, _log.logContent)(rawSourceContent);
  }
}