'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = function () {
  var userScripts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var combinedScripts = (0, _lodash2.default)([].concat(_toConsumableArray(Object.keys(saguiScripts)), _toConsumableArray(Object.keys(userScripts)))).sort();

  return combinedScripts.reduce(function (scripts, key) {
    var isCustomScript = !saguiScripts[key];

    if (isCustomScript) {
      return _extends({}, scripts, { [key]: userScripts[key] });
    }

    var isMissingScript = !userScripts[key];
    var hasAnOutdatedScript = saguiScripts[key].indexOf(userScripts[key]) !== -1;
    var latestScript = saguiScripts[key][saguiScripts[key].length - 1];
    var latestScriptIsDefined = latestScript !== undefined;

    if (hasAnOutdatedScript && latestScriptIsDefined || isMissingScript && latestScriptIsDefined) {
      return _extends({}, scripts, { [key]: latestScript });
    }

    if (!hasAnOutdatedScript && !isMissingScript) {
      return _extends({}, scripts, { [key]: userScripts[key] });
    }

    return scripts;
  }, {});
};

var _lodash = require('lodash.uniq');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * List of scripts that provide an upgrade path
 * Each "script key" has a list of all past and
 * current value for the script.
 *
 * If we need to update a script, simply append to the end
 * of the list.
 */
var saguiScripts = {
  'format': ['sagui format'],
  'build': ['sagui build'],
  'develop': ['sagui develop --port 3000', undefined],
  'dist': ['NODE_ENV=production sagui build --optimize', 'cross-env NODE_ENV=production sagui build --optimize', 'sagui build --optimize', 'sagui dist'],
  'start': ['npm run develop', 'sagui develop --port 3000'],
  'test': ['echo "Error: no test specified" && exit 1', 'npm run test:lint && npm run test:unit', 'npm run test:lint && npm run test:typecheck && npm run test:unit', 'sagui test'],
  'test:coverage': ['npm run test:unit -- --coverage', undefined],
  'test:lint': ['sagui lint', 'sagui test:lint', undefined, 'sagui test:lint'],
  'test:typecheck': ['sagui typecheck', 'sagui test:typecheck'],
  'test:unit': ['NODE_ENV=test sagui test', 'cross-env NODE_ENV=test sagui test', 'cross-env NODE_ENV=test sagui test:unit', 'sagui test:unit', 'sagui test:unit --coverage'],
  'test:unit:watch': ['npm run test:unit -- --watch', 'cross-env NODE_ENV=test sagui test:unit --watch', 'sagui test:unit --watch']
};